// for HELTEC WiFi LoRa 32;    
// BT name : HELTEC_WX_TX_1

////////////////////////////////////////////////////

const byte lora_PNSS = 18;    // pin number where the NSS line for the LoRa device is connected.
const byte lora_PReset = 14;  // pin where LoRa device reset line is connected
const byte PLED1 = 2;         // pin number for LED on Tracker

String Outstring="";     

#include <SPI.h>
#include "LoRaTX.h"
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
#include "BluetoothSerial.h"

// DISPLAY address
#define SSD1306_ADDRESS 0x3C
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
#define OLED_RESET 16 

// BME280
#define SEALEVELPRESSURE_HPA (1019) 
Adafruit_BME280 bme;
float SLpressure_hPa;
float wind, temp, tempf, humi, alti, pres;
int cnt=1;
int wind_cnt, wind2;
int anopin = 12; //PIN for anemometer input

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

BluetoothSerial bluetooth;

/////////////////////////////////////////////////////////////////////////////////////////
void setup()
{
  bluetooth.begin("HELTEC_WX_TX_1");
  Serial.begin(9600);                                                              // Serial console ouput
  delay(3000);
  Wire.begin(4,15); 
  
  bool bme_status;
  bme_status = bme.begin(0x76);  //address either 0x76 or 0x77
  if (!bme_status) {
  Serial.print("No valid BME280 found"); }

  display.begin(SSD1306_SWITCHCAPVCC, 0x3c); 
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(2);
  display.setCursor(0,0);
  display.println("  LoRa-WX ");
  display.setCursor(0,20);
  display.println("  Encoder");
  display.setTextSize(1);
  display.setCursor(25,55);
  display.println("(C)DJ7OO 2020");
  display.display();  
  delay(3000);
  
  pinMode(lora_PReset, OUTPUT);     // RFM98 reset line
  digitalWrite(lora_PReset, LOW);   // Reset RFM98
  pinMode (lora_PNSS, OUTPUT);      // set the slaveSelectPin as an output:
  digitalWrite(lora_PNSS, HIGH);
  pinMode(PLED1, OUTPUT);                                                          // for shield LED
  
  SPI.begin();                                                                     // initialize SPI:
  SPI.setClockDivider(SPI_CLOCK_DIV2);
  SPI.setDataMode(SPI_MODE0);
  SPI.setBitOrder(MSBFIRST);
   
   lora_ResetDev();                                                                 // Reset the device
   lora_Setup();                                                                    // Do the initial LoRa Setup

// LoRa frequency calculation (sample for 434.4 MHz): 
// ------------------------------------
// 434400000/61.03515625 = 71172096
// 71172096 (DEC) = 6C 99 99 (HEX)  
// 6C 99 99 (HEX) = 108 153 153 (DEC)  

   lora_SetFreq(108, 113, 153);  //433.775 MHz                                                    // Set the LoRa frequency, 434.400 Mhz
// lora_SetFreq(108, 153, 153);  //434.400 MHz                                                  // Set the LoRa frequency, 434.400 Mhz
}

///////////////////////////////////////////////////////////////////////////////////////
void loop()
{
  bme.takeForcedMeasurement();
  temp = bme.readTemperature();
  humi = bme.readHumidity();
  alti = bme.readAltitude(SEALEVELPRESSURE_HPA);
  pres = bme.readPressure()/100.0F;
 
   byte i;
   byte ltemp;
   tempf=(temp*1.8)+32; // celsius to fahrenheit
   meassure_wind();     // out is wind in Km/h
   wind2 = wind/3.6;    // wind2 is in mps

   Outstring = "DJ7OO-12>APRS:!4957.60N/00812.00E_";   // anpassen
   Outstring += "000/";
   if(wind2<100) { Outstring += "0"; }
   if(wind2<10)  { Outstring += "0"; }
   Outstring += wind2;
   Outstring += ("g...");
   Outstring += ("t");
   if(tempf<100) { Outstring += "0"; }
   if(tempf<10) { Outstring += "0"; }
   Outstring += String(tempf,0);
   Outstring += ("h");
   Outstring += String(humi,0);
   Outstring += ("b");
   if(pres<1000) { Outstring += "0"; }
   Outstring += String(pres,0);
   Outstring += ("0");
   Outstring += (" WX-Mainz");

// Serial.print("sending: ");
// Serial.println(Outstring);

    ltemp = Outstring.length();
    lora_SetModem(lora_BW125, lora_SF12, lora_CR4_5, lora_Explicit, lora_LowDoptON);		// Setup the LoRa modem parameters
    lora_PrintModem();                                                                  // Print the modem parameters
    lora_TXStart = 0;
    lora_TXEnd = 0;
    for (i = 0; i <= ltemp; i++)
    {
    lora_TXBUFF[i] = Outstring.charAt(i);
    }
    i--;
    lora_TXEnd = i;    
    digitalWrite(PLED1, HIGH);  // LED ON on during sending
    
    display.clearDisplay();
    display.setTextColor(WHITE);
    display.setTextSize(3);
    display.setCursor(18,10);
    display.print("ON AIR");
    display.setTextSize(2);
    display.setCursor(50,40);
    if(cnt<1000) {display.print("0");}
    if(cnt<100)  {display.print("0");}
    if(cnt<10)   {display.print("0");}
    display.print(cnt);
    display.display();  

    lora_Send(lora_TXStart, lora_TXEnd, 60, 255, 1, 10, 17);	
    digitalWrite(PLED1, LOW);   // LED OFF after sending
    lora_TXBuffPrint(0);
    make_display();
    Outstring = "";
    cnt=cnt+1;
    if(cnt>9999){cnt=0;}

    int z,calc;
//   calc = 1800;  // 1800sec. = 30 Min.
//   calc = 900;   // 900sec.  = 15 Min.
//   calc = 600;   // 600sec.  = 10 Min.
//   calc = 300;   // 300sec.  = 5 Min.
     calc = 60;    //  60sec.  = 1 Min.
//   calc = 30;    //  60sec.  = 30 Sec.

  Serial.println(" ");
  Serial.print("TX-Delay: "),
  Serial.print(calc),
  Serial.println("sec.");
  for(z=1; z<(calc); z++)  { delay(1000); }  
}

/////////////////////////////////////////////
void make_display()
{
  Serial.println(" ");
  Serial.print("Temperature = ");
  Serial.print(temp,1);
  Serial.println("°C");
  Serial.print("Humidiy     = ");
  Serial.print(humi,0);
  Serial.println("%");
  Serial.print("Pressure    = ");
  Serial.print(pres,0);
  Serial.println("hPa");
  Serial.print("Altitude    = ");
  Serial.print(alti,0);
  Serial.println("m");
  Serial.print("Wind        = ");
  Serial.print(wind,0);
  Serial.println("Km/h");
  
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(1);
  display.setCursor(0,0);
  display.print("Temperature: ");
  display.print(temp,1);
  display.println(" C");
  display.setCursor(0,12);
  display.print("Humidity   : ");
  display.print(humi,0);
  display.println(" %");
  display.setCursor(0,24);
  display.print("Altitude   : ");
  display.print(alti,0);
  display.println(" m");
  display.setCursor(0,36);
  display.print("Pressure   : ");
  display.print(pres,0);
  display.println(" hPa");
  display.setCursor(0,48);
  display.print("Wind       : ");
  display.print(wind,0);
  display.println(" Km/h");
  display.display();  

// sending via Bluetooth 
  bluetooth.print("Temperature: ");
  bluetooth.print(temp,1);
  bluetooth.println(" C");
  bluetooth.print("Humidity   : ");
  bluetooth.print(humi,0);
  bluetooth.println(" %");
  bluetooth.print("Altitude   : ");
  bluetooth.print(alti,0);
  bluetooth.println(" m");
  bluetooth.print("Pressure   : ");
  bluetooth.print(pres,0);
  bluetooth.println(" hPa");
  bluetooth.print("Wind       : ");
  bluetooth.print(wind,0);
  bluetooth.println(" Km/h");
  bluetooth.println(" ");
}

//////////////////////////////////////////////////////////
void meassure_wind() {
  wind_cnt = 0;
  attachInterrupt(digitalPinToInterrupt(anopin), intrrupt_cnt, RISING);
  delay(3000); //Time for measure counts
  detachInterrupt(digitalPinToInterrupt(anopin));
  wind = ((float)wind_cnt / (float)3 * 2.4) / 2; //Convert counts & time to km/h
}

/////////////////////////////////////////////////////
void intrrupt_cnt() {//On count detected
  wind_cnt++;//Add one count
}
